#ifndef CoCoA_NumTheory_H
#define CoCoA_NumTheory_H

//   Copyright (c)  2009  John Abbott

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.
//   -------------------------------------------------------------------------

#include "CoCoA/MachineInteger.H"
#include "CoCoA/factorization.H"
#include "CoCoA/ZZ.H"

namespace CoCoA
{

  // Frankly 8 separate cases seems a bit excessive!
  long PowerMod(const MachineInteger& base, const MachineInteger& exp, const MachineInteger& modulus);
  long PowerMod(const MachineInteger& base, const ZZ& exp, const MachineInteger& modulus);
  long PowerMod(const ZZ& base, const MachineInteger& exp, const MachineInteger& modulus);
  long PowerMod(const ZZ& base, const ZZ& exp, const MachineInteger& modulus);
  ZZ PowerMod(const MachineInteger& base, const MachineInteger& exp, const ZZ& modulus);
  ZZ PowerMod(const ZZ& base, const MachineInteger& exp, const ZZ& modulus);
  ZZ PowerMod(const MachineInteger& base, const ZZ& exp, const ZZ& modulus);
  ZZ PowerMod(const ZZ& base, const ZZ& exp, const ZZ& modulus);

  long gcd(const MachineInteger& a, const MachineInteger& b);
  const ZZ gcd(const ZZ& A, const MachineInteger& b);
  const ZZ gcd(const MachineInteger& a, const ZZ& B);
  const ZZ gcd(const ZZ& A, const ZZ& B);

  long ExtGcd(long& CofacA, long& CofacB, const MachineInteger& a, const MachineInteger& b);
//???? signature???  ZZ ExtGcd(ZZ& CoFacA, ZZ& CofacB, const MachineInteger& A, const ZZ& B);
//???? signature???  ZZ ExtGcd(ZZ& CoFacA, ZZ& CofacB, const ZZ& A, const MachineInteger& B);
  ZZ ExtGcd(ZZ& CoFacA, ZZ& CofacB, const ZZ& A, const ZZ& B);

  long InvMod(const MachineInteger& r, const MachineInteger& m);
  long InvMod(const ZZ& r, const MachineInteger& m);
  ZZ InvMod(const MachineInteger& r, const ZZ& m);
  ZZ InvMod(const ZZ& r, const ZZ& m);

  long lcm(const MachineInteger& a, const MachineInteger& b);
  const ZZ lcm(const ZZ& A, const MachineInteger& b);
  const ZZ lcm(const MachineInteger& a, const ZZ& B);
  const ZZ lcm(const ZZ& a, const ZZ& b);


  bool IsPrime(const MachineInteger& n);
  bool IsPrime(const ZZ& N); // might be very slow!!
  bool IsProbPrime(const MachineInteger& n, const MachineInteger& NumIters = 25); // probably not very useful
  bool IsProbPrime(const ZZ& N, const MachineInteger& NumIters = 25);

  long NextPrime(const MachineInteger& n); ///< next prime number after n or 0 if there is none, must have n>0
  long PrevPrime(const MachineInteger& n); ///< previous prime number before n or 0 if there is none, must have n>0

  // !!No NextPrime for ZZ!!  Use NextProbPrime and call IsPrime if you want a (costly) guarantee of primality.
  ZZ NextProbPrime(ZZ N, const MachineInteger& NumIters = 25); ///< next probable prime number after N, requires N > 1
  ZZ PrevProbPrime(ZZ N, const MachineInteger& NumIters = 25); ///< previous probable prime number before N, requires N > 1


  factorization<long> SmoothFactor(const MachineInteger& N, const MachineInteger& TrialLimit);
  factorization<ZZ>   SmoothFactor(const ZZ& N, const MachineInteger& TrialLimit);
  factorization<ZZ>   SmoothFactor(const ZZ& N, const ZZ& TrialLimit);

  factorization<long> factor(const MachineInteger& N);
  factorization<ZZ>   factor(const ZZ& N);

  long EulerPhi(const MachineInteger& n);
  ZZ EulerPhi(const ZZ& N);

  long MultiplicativeOrderMod(const MachineInteger& residue, const MachineInteger& modulus);
  long MultiplicativeOrderMod(const ZZ& residue, const MachineInteger& modulus);
  ZZ MultiplicativeOrderMod(const MachineInteger& residue, const ZZ& modulus);
  ZZ MultiplicativeOrderMod(const ZZ& residue, const ZZ& modulus);
  int PrimitiveRoot(const MachineInteger& p);
  int PrimitiveRoot(const ZZ& P);

} // end of namespace CoCoA



// RCS header/log in the next few lines
// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/include/CoCoA/NumTheory.H,v 1.7 2011/03/23 21:00:46 abbott Exp $
// $Log: NumTheory.H,v $
// Revision 1.7  2011/03/23 21:00:46  abbott
// Removed FindPrimRoot from NumTheory.H because it was already
// available as PrimitiveRoot (a better name).
// Updated documentation for NumTheory.
//
// Revision 1.6  2011/03/22 20:17:18  abbott
// Added fn FindPrimRoot.
// Merged impls from obsolescent SmallPrime.C.
//
// Revision 1.5  2011/03/16 13:26:36  abbott
// Removed all "unsigned" from fn interfaces, and many unsigned from inside fn impls.
//
// Revision 1.4  2010/03/03 10:43:34  abbott
// Added PrimitiveRoot for big primes (might be very slow).
// Added MultiplicativeOrderMod (currently very SLUGGY implementation).
//
// Revision 1.3  2009/12/29 22:44:32  abbott
// Removed buggy proxy class ZZ::rtn.
// Consequent changes for function prototypes also in NumTheory.
// Corrected some minor buglets in NumTheory.
//
// Revision 1.2  2009/07/02 16:28:10  abbott
// Fairly extensive change to NumTheory (at least internally and philosophically).
// Improved and cleaned NumTheory.  Added documentation.
// Clarified the exact behaviour of most functions.
//
// Revision 1.1  2009/06/05 12:14:55  abbott
// Major change:
//   created new files NumTheory.H/C  which contain basic number theory operations
//   removed several basic number theory operations from ZZ.H/C
//   removed gcd from MachineInteger.H/C
//   changed names of some basic fns:
//      IsPPrime -> IsProbPrime
//      invmod -> InvMod    (changed signature too)
//      powermod -> PowerMod  (changed signature too)
//   added new fns
//      NextProbPrime & PrevProbPrime
//   consequent changes to other code and tests and examples
//
//

#endif
