//   Copyright (c)  2010  John Abbott

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.


#include "CoCoA/BuildInfo.H"
#include "CoCoA/error.H"
#include "CoCoA/GlobalManager.H"
#include "CoCoA/NumTheory.H"
#include "CoCoA/QQ.H"

#include <iostream>
using std::cerr;
using std::endl;

using namespace CoCoA;

// Handy macro for making assertions.
#define TEST_ASSERT(cond)   do { if (!(cond)) CoCoA::AssertionFailed(#cond, __FILE__, __LINE__); } while(0)

void program()
{
  // This test checks that the QQ::AlreadyReduced flag works correctly.
  GlobalManager CoCoAFoundations;

  const int NMAX = 99;
  for (int n=-NMAX; n <= NMAX; ++n)
    for (int d=-NMAX; d <= NMAX; ++d)
    {
      if (d == 0 || gcd(n,d) > 1) continue;
      const QQ q = QQ(n,d);
      TEST_ASSERT(q == QQ(n,d,        QQ::AlreadyReduced));
      TEST_ASSERT(q == QQ(ZZ(n),d,    QQ::AlreadyReduced));
      TEST_ASSERT(q == QQ(n,ZZ(d),    QQ::AlreadyReduced));
      TEST_ASSERT(q == QQ(ZZ(n),ZZ(d),QQ::AlreadyReduced));

      TEST_ASSERT(den(QQ(n,d,        QQ::AlreadyReduced)) > 0);
      TEST_ASSERT(den(QQ(ZZ(n),d,    QQ::AlreadyReduced)) > 0);
      TEST_ASSERT(den(QQ(n,ZZ(d),    QQ::AlreadyReduced)) > 0);
      TEST_ASSERT(den(QQ(ZZ(n),ZZ(d),QQ::AlreadyReduced)) > 0);

      if (n != 0)
      {
        const QQ recip = QQ(d,n,QQ::AlreadyReduced);
        TEST_ASSERT(power(q,-1) == recip);
        for (int e=1; e < 9; ++e)
          TEST_ASSERT(power(q,-e) == power(recip,e));
      }
    }
}


// Use main() to handle any uncaught exceptions and warn the user about them.
int main()
{
  try
  {
    program();
    return 0;
  }
  catch (const CoCoA::ErrorInfo& err)
  {
    cerr << "***ERROR***  UNCAUGHT CoCoA Error";
    ANNOUNCE(cerr, err);
  }
  catch (const std::exception& exc)
  {
    cerr << "***ERROR***  UNCAUGHT std::exception: " << exc.what() << endl;
  }
  catch(...)
  {
    cerr << "***ERROR***  UNCAUGHT UNKNOWN EXCEPTION" << endl;
  }

  BuildInfo::PrintAll(cerr);
  return 1;
}
